﻿Imports System.Xml

Public Interface IDBLink
    Function GetExpirationDate(ByVal InstallationKey As String) As Date
    Sub SetExpirationDate(ByVal InstallationKey As String, ByVal NewDate As Date)
    Function ExtendDemo(ByVal UniqueInstallGUID As String, ByVal NewDate As Date) As String
    Function StripXmlSignature(ByVal xmldocstring As String) As String
    Function GetCurrentDemoExpirationDate(ByVal UniqueInstallGUID As String) As Date
    Sub SetConcurrencyInfo(ByVal InstallationKey As String, ByVal Server As String, ByVal count As Integer)
    Function GetConcurrencyInfo(ByVal InstallationKey As String, ByRef server As String, ByRef count As Integer) As Boolean
End Interface

#If SERVER Then
Public Class dblink
    Implements IDBLink

    Implements IDisposable


    Public Shared Initialized As Boolean

    Private theconnection As IDbConnection
    Private theadapter As IDbDataAdapter
    Private thecommand As IDbCommand ' Generic command for use as needed
    Private thebuilder As Object


    Public Sub New(ByVal ConnectionString As String, ByVal dbtype As String)
        ' Generalize the access
        Select Case dbtype.ToLower
            Case "sql"
                theconnection = New Data.SqlClient.SqlConnection(ConnectionString)
                theadapter = New Data.SqlClient.SqlDataAdapter()
                theadapter.SelectCommand = New SqlClient.SqlCommand("Select * FROM IKeyExtensions", CType(theconnection, SqlClient.SqlConnection))
                thecommand = New SqlClient.SqlCommand("Select * FROM Expirations", CType(theconnection, SqlClient.SqlConnection))
                thebuilder = New Data.SqlClient.SqlCommandBuilder(CType(theadapter, SqlClient.SqlDataAdapter))

            Case "oledb"
                theconnection = New Data.OleDb.OleDbConnection(ConnectionString)
                theadapter = New Data.OleDb.OleDbDataAdapter()
                theadapter.SelectCommand = New OleDb.OleDbCommand("Select * FROM IKeyExtensions", CType(theconnection, OleDb.OleDbConnection))
                thecommand = New OleDb.OleDbCommand("Select * FROM Expirations", CType(theconnection, OleDb.OleDbConnection))
                thebuilder = New Data.OleDb.OleDbCommandBuilder(CType(theadapter, OleDb.OleDbDataAdapter))

            Case "odbc"
                theconnection = New Data.Odbc.OdbcConnection(ConnectionString)
                theadapter = New Data.Odbc.OdbcDataAdapter()
                theadapter.SelectCommand = New Odbc.OdbcCommand("Select * FROM IKeyExtensions", CType(theconnection, Odbc.OdbcConnection))
                thecommand = New Odbc.OdbcCommand("Select * FROM IKeyExtensions", CType(theconnection, Odbc.OdbcConnection))
                thebuilder = New Data.Odbc.OdbcCommandBuilder(CType(theadapter, Odbc.OdbcDataAdapter))
        End Select
        If Not Initialized Then
            ' Check the database
            thecommand.CommandText = "SELECT TOP 1 * FROM IKeyExtensions ORDER BY 'ID'"
            Try
                theconnection.Open()
                thecommand.ExecuteScalar()
            Catch ex As Exception
                ' Try creating the table
                Trace.WriteLine("Exception: " & ex.Message & vbCrLf & " Attempting to create table")
                Try
                    Dim expbuilder As New System.Text.StringBuilder
                    expbuilder.Append("CREATE TABLE IKeyExtensions ( ")
                    expbuilder.Append("ID int IDENTITY PRIMARY KEY,")
                    expbuilder.Append("InstallationCode varchar(32) UNIQUE NOT NULL, ")
                    expbuilder.Append("ConcurrencyServer varchar(255) NULL,")
                    expbuilder.Append("ConcurrencyCount INT NULL, ")
                    expbuilder.Append("ExpirationDate datetime NULL")
                    expbuilder.Append(")")
                    thecommand.CommandText = expbuilder.ToString
                    thecommand.ExecuteScalar()
                Catch ex2 As Exception
                    Trace.WriteLine("Unable to create table: " & ex.Message)
                End Try
            End Try
            If theconnection.State = ConnectionState.Open Then theconnection.Close()

            Initialized = True
        End If


    End Sub

    Private Sub RefreshTheBuilder()
        thebuilder.GetType.InvokeMember("RefreshSchema", Reflection.BindingFlags.Instance Or Reflection.BindingFlags.InvokeMethod Or Reflection.BindingFlags.Public, Nothing, thebuilder, Nothing)
    End Sub

    Private Function GetRowForKey(ByVal InstallationKey As String) As DataSet
        theconnection.Open()
        theadapter.SelectCommand.CommandText = "Select * FROM IKeyExtensions WHERE InstallationCode = '" & InstallationKey & "'"
        RefreshTheBuilder()
        Dim ds As New DataSet
        theadapter.Fill(ds)
        Return ds
    End Function

    Public Function GetExpirationDate(ByVal InstallationKey As String) As Date Implements IDBLink.GetExpirationDate
        Try
            Dim ds As New DataSet
            ds = GetRowForKey(InstallationKey)
            If ds.Tables(0).Rows.Count = 0 Then Return Date.MaxValue ' No expiration
            If IsDBNull(ds.Tables(0).Rows(0).Item("ExpirationDate")) Then Return Date.MaxValue
            Return CDate(ds.Tables(0).Rows(0).Item("ExpirationDate"))
        Catch ex As Exception
            Trace.WriteLine("Error in GetExpirationDate: " & ex.Message)
            Throw (ex)
        Finally
            If theconnection.State = ConnectionState.Open Then theconnection.Close()
        End Try
    End Function

    Public Sub SetExpirationDate(ByVal InstallationKey As String, ByVal NewDate As Date) Implements IDBLink.SetExpirationDate
        Try
            Dim ds As New DataSet
            ds = GetRowForKey(InstallationKey)
            Dim therow As DataRow
            If ds.Tables(0).Rows.Count = 0 Then
                therow = ds.Tables(0).NewRow()
                ds.Tables(0).Rows.Add(therow)
            Else
                therow = ds.Tables(0).Rows(0)
            End If
            therow("InstallationCode") = InstallationKey
            therow("ExpirationDate") = NewDate
            theadapter.Update(ds)

        Catch ex As Exception
            Trace.WriteLine("Error in SetExpirationDate: " & ex.Message)
            Throw (ex)
        Finally
            If theconnection.State = ConnectionState.Open Then theconnection.Close()
        End Try

    End Sub


    Function ExtendDemo(ByVal UniqueInstallGUID As String, ByVal NewDate As Date) As String Implements IDBLink.ExtendDemo
        Try
            theconnection.Open()
            theadapter.SelectCommand.CommandText = "Select * FROM UniqueInstalls WHERE UniqueInstallGUID = '" & UniqueInstallGUID & "'"
            RefreshTheBuilder()
            Dim ds As New DataSet
            Dim therow As DataRow
            If theadapter.Fill(ds) = 0 Then
                Return ("Unable to find specified UniqueIntallGUID")
            Else
                therow = ds.Tables(0).Rows(0)
                therow("DemoExpiration") = NewDate
                theadapter.Update(ds)
            End If
        Catch ex As Exception
            Return ("Error: " & ex.Message)
        Finally
            If theconnection.State = ConnectionState.Open Then theconnection.Close()
        End Try
        Return ("Success")
    End Function

    Function StripXmlSignature(ByVal xmldocstring As String) As String Implements IDBLink.StripXmlSignature
        Try
            Dim xdoc As New XmlDocument()
            xdoc.LoadXml(xmldocstring)
            Dim objnode As XmlNode = xdoc.GetElementsByTagName("Object").Item(0)
            Dim newxml As New XmlDocument
            newxml.AppendChild(newxml.CreateElement("LicenseInfo"))
            newxml.FirstChild.InnerXml = objnode.InnerXml
            Dim st As New IO.StringWriter()
            newxml.Save(st)
            Return st.ToString

        Catch ex As Exception
            Trace.WriteLine("Unable to strip XML signature: " & ex.Message)
            Return Nothing
        End Try
    End Function

    Function GetCurrentDemoExpirationDate(ByVal UniqueInstallGUID As String) As Date Implements IDBLink.GetCurrentDemoExpirationDate
        Try
            theconnection.Open()
            theadapter.SelectCommand.CommandText = "Select * FROM UniqueInstalls WHERE UniqueInstallGUID = '" & UniqueInstallGUID & "'"
            RefreshTheBuilder()
            Dim ds As New DataSet
            Dim therow As DataRow
            If theadapter.Fill(ds) = 0 Then
                Return Nothing
            Else
                therow = ds.Tables(0).Rows(0)
                Return CDate(therow("DemoExpiration"))
            End If
        Catch ex As Exception
            Trace.WriteLine("Error in GetCurrentExpirationDate: " & ex.Message)
            Return Nothing
        Finally
            If theconnection.State = ConnectionState.Open Then theconnection.Close()
        End Try
    End Function

    Public Function GetConcurrencyInfo(ByVal InstallationKey As String, ByRef server As String, ByRef count As Integer) As Boolean Implements IDBLink.GetConcurrencyInfo
        Try
            Dim ds As New DataSet
            ds = GetRowForKey(InstallationKey)
            If ds.Tables(0).Rows.Count = 0 Then Return False ' No expiration
            With ds.Tables(0).Rows(0)
                If IsDBNull(.Item("ConcurrencyServer")) OrElse IsDBNull(.Item("ConcurrencyCount")) Then Return False
                server = CStr(.Item("ConcurrencyServer"))
                count = CInt(.Item("ConcurrencyCount"))
                Return (True)
            End With
        Catch ex As Exception
            Trace.WriteLine("Error in GetConcurrencyInfo: " & ex.Message)
            Throw (ex)
        Finally
            If theconnection.State = ConnectionState.Open Then theconnection.Close()
        End Try

    End Function

    Public Sub SetConcurrencyInfo(ByVal InstallationKey As String, ByVal Server As String, ByVal count As Integer) Implements IDBLink.SetConcurrencyInfo
        Try
            Dim ds As New DataSet
            ds = GetRowForKey(InstallationKey)
            Dim therow As DataRow
            If ds.Tables(0).Rows.Count = 0 Then
                therow = ds.Tables(0).NewRow()
                ds.Tables(0).Rows.Add(therow)
            Else
                therow = ds.Tables(0).Rows(0)
            End If
            therow("InstallationCode") = InstallationKey
            therow("ConcurrencyServer") = Server
            therow("ConcurrencyCount") = count
            theadapter.Update(ds)

        Catch ex As Exception
            Trace.WriteLine("Error in SetExpirationDate: " & ex.Message)
            Throw (ex)
        Finally
            If theconnection.State = ConnectionState.Open Then theconnection.Close()
        End Try
    End Sub

#Region " IDisposable Support "

    Private disposedValue As Boolean = False        ' To detect redundant calls

    ' IDisposable
    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not Me.disposedValue Then
            If disposing Then
                If thecommand IsNot Nothing Then thecommand.Dispose()
                If thebuilder IsNot Nothing Then thebuilder.GetType.InvokeMember("Dispose", Reflection.BindingFlags.Instance Or Reflection.BindingFlags.InvokeMethod Or Reflection.BindingFlags.Public, Nothing, thebuilder, Nothing)
            End If

        End If
        Me.disposedValue = True
    End Sub


    ' This code added by Visual Basic to correctly implement the disposable pattern.
    Public Sub Dispose() Implements IDisposable.Dispose
        ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region


End Class

#End If
